MAX_SAMPLES = 10000
RENDERED_OBJECT_INFO_DEFINITION_ID = "659c6e36-f9f8-4dd6-9651-4a80e51eabc4"
USER_PARAMETERS_DEFINITION_ID = "3f06bcec-1f23-4387-a1fd-5af54ee29c16"
FOREGROUND_PLACEMENT_INFO_DEFINITION_ID = "061e08cc-4428-4926-9933-a6732524b52b"
LIGHTING_INFO_DEFINITION_ID = "939248ee-668a-4e98-8e79-e7909f034a47"
BOUNDING_BOX_2D_DEFINITION_ID = "c31620e3-55ff-4af6-ae86-884aa0daa9b2"
